@@ -69,18 +69,13 @@ import {
 
 const loadBenchmarkDataset = (( ) => {
     let datasetPromise;
-    let ttlTimer;
+
+    const ttlTimer = vAPI.defer.create(( ) => {
+        datasetPromise = undefined;
+    });
 
     return function() {
-        if ( ttlTimer !== undefined ) {
-            clearTimeout(ttlTimer);
-            ttlTimer = undefined;
-        }
-
-        setTimeout(( ) => {
-            ttlTimer = undefined;
-            datasetPromise = undefined;
-        }, 5 * 60 * 1000);
+        ttlTimer.offon({ min: 5 });
 
         if ( datasetPromise !== undefined ) {
             return datasetPromise;
@@ -94,14 +89,30 @@ const loadBenchmarkDataset = (( ) => {
         console.info(`Loading benchmark dataset...`);
         datasetPromise = io.fetchText(datasetURL).then(details => {
             console.info(`Parsing benchmark dataset...`);
-            const requests = [];
-            const lineIter = new LineIterator(details.content);
-            while ( lineIter.eot() === false ) {
-                let request;
+            let requests = [];
+            if ( details.content.startsWith('[') ) {
                 try {
-                    request = JSON.parse(lineIter.next());
+                    requests = JSON.parse(details.content);
                 } catch(ex) {
                 }
+            } else {
+                const lineIter = new LineIterator(details.content);
+                const parsed = [];
+                while ( lineIter.eot() === false ) {
+                    const line = lineIter.next().trim();
+                    if ( line === '' ) { continue; }
+                    try {
+                        parsed.push(JSON.parse(line));
+                    } catch(ex) {
+                        parsed.length = 0;
+                        break;
+                    }
+                }
+                requests = parsed;
+            }
+            if ( requests.length === 0 ) { return; }
+            const out = [];
+            for ( const request of requests ) {
                 if ( request instanceof Object === false ) { continue; }
                 if ( !request.frameUrl || !request.url ) { continue; }
                 if ( request.cpt === 'document' ) {
@@ -109,9 +120,9 @@ const loadBenchmarkDataset = (( ) => {
                 } else if ( request.cpt === 'xhr' ) {
                     request.cpt = 'xmlhttprequest';
                 }
-                requests.push(request);
+                out.push(request);
             }
-            return requests;
+            return out;
         }).catch(details => {
             console.info(`Not found: ${details.url}`);
             datasetPromise = undefined;
@@ -171,12 +182,13 @@ const loadBenchmarkDataset = (( ) => {
         else if ( r === 2 ) { allowCount += 1; }
         if ( r !== 1 ) {
             if ( staticNetFilteringEngine.hasQuery(fctxt) ) {
-                staticNetFilteringEngine.filterQuery(fctxt, 'queryprune');
+                staticNetFilteringEngine.filterQuery(fctxt, 'removeparam');
             }
             if ( fctxt.type === 'main_frame' || fctxt.type === 'sub_frame' ) {
                 staticNetFilteringEngine.matchAndFetchModifiers(fctxt, 'csp');
+                staticNetFilteringEngine.matchAndFetchModifiers(fctxt, 'permissions');
             }
-            staticNetFilteringEngine.matchHeaders(fctxt, []);
+            //staticNetFilteringEngine.matchHeaders(fctxt, []);
         } else if ( redirectEngine !== undefined ) {
             staticNetFilteringEngine.redirectRequest(redirectEngine, fctxt);
         }
