@@ -35,6 +35,116 @@ vAPI.T0 = Date.now();
 
 vAPI.setTimeout = vAPI.setTimeout || self.setTimeout.bind(self);
 
+vAPI.defer = {
+    create(callback) {
+        return new this.Client(callback);
+    },
+    once(delay, ...args) {
+        const delayInMs = vAPI.defer.normalizeDelay(delay);
+        return new Promise(resolve => {
+            vAPI.setTimeout(
+                (...args) => { resolve(...args); },
+                delayInMs,
+                ...args
+            );
+        });
+    },
+    Client: class {
+        constructor(callback) {
+            this.timer = null;
+            this.type = 0;
+            this.callback = callback;
+        }
+        on(delay, ...args) {
+            if ( this.timer !== null ) { return; }
+            const delayInMs = vAPI.defer.normalizeDelay(delay);
+            this.type = 0;
+            this.timer = vAPI.setTimeout(( ) => {
+                this.timer = null;
+                this.callback(...args);
+            }, delayInMs || 1);
+        }
+        offon(delay, ...args) {
+            this.off();
+            this.on(delay, ...args);
+        }
+        onvsync(delay, ...args) {
+            if ( this.timer !== null ) { return; }
+            const delayInMs = vAPI.defer.normalizeDelay(delay);
+            if ( delayInMs !== 0 ) {
+                this.type = 0;
+                this.timer = vAPI.setTimeout(( ) => {
+                    this.timer = null;
+                    this.onraf(...args);
+                }, delayInMs);
+            } else {
+                this.onraf(...args);
+            }
+        }
+        onidle(delay, options, ...args) {
+            if ( this.timer !== null ) { return; }
+            const delayInMs = vAPI.defer.normalizeDelay(delay);
+            if ( delayInMs !== 0 ) {
+                this.type = 0;
+                this.timer = vAPI.setTimeout(( ) => {
+                    this.timer = null;
+                    this.onric(options, ...args);
+                }, delayInMs);
+            } else {
+                this.onric(options, ...args);
+            }
+        }
+        off() {
+            if ( this.timer === null ) { return; }
+            switch ( this.type ) {
+            case 0:
+                self.clearTimeout(this.timer);
+                break;
+            case 1:
+                self.cancelAnimationFrame(this.timer);
+                break;
+            case 2:
+                self.cancelIdleCallback(this.timer);
+                break;
+            default:
+                break;
+            }
+            this.timer = null;
+        }
+        onraf(...args) {
+            if ( this.timer !== null ) { return; }
+            this.type = 1;
+            this.timer = requestAnimationFrame(( ) => {
+                this.timer = null;
+                this.callback(...args);
+            });
+        }
+        onric(options, ...args) {
+            if ( this.timer !== null ) { return; }
+            this.type = 2;
+            this.timer = self.requestIdleCallback(deadline => {
+                this.timer = null;
+                this.callback(deadline, ...args);
+            }, options);
+        }
+        ongoing() {
+            return this.timer !== null;
+        }
+    },
+    normalizeDelay(delay = 0) {
+        if ( typeof delay === 'object' ) {
+            if ( delay.sec !== undefined ) {
+                return delay.sec * 1000;
+            } else if ( delay.min !== undefined ) {
+                return delay.min * 60000;
+            } else if ( delay.hr !== undefined ) {
+                return delay.hr * 3600000;
+            }
+        }
+        return delay;
+    }
+};
+
 /******************************************************************************/
 
 vAPI.webextFlavor = {
