@@ -73,7 +73,7 @@
         enableVideoAds: noopfn,
         get: noopnullfn,
         getAttributeKeys: nooparrayfn,
-        getTargeting: noopfn,
+        getTargeting: nooparrayfn,
         getTargetingKeys: nooparrayfn,
         getSlots: nooparrayfn,
         refresh: noopfn,
@@ -85,6 +85,7 @@
         setForceSafeFrame: noopthisfn,
         setLocation: noopthisfn,
         setPublisherProvidedId: noopthisfn,
+        setPrivacySettings: noopthisfn,
         setRequestNonPersonalizedAds: noopthisfn,
         setSafeFrameConfig: noopthisfn,
         setTagForChildDirectedTreatment: noopthisfn,
@@ -119,6 +120,7 @@
     p.setClickUrl = noopthisfn;
     p.setCollapseEmptyDiv = noopthisfn;
     p.setTargeting = noopthisfn;
+    p.updateTargetingFromMap = noopthisfn;
     //
     const gpt = window.googletag || {};
     const cmd = gpt.cmd || [];
